#!/usr/bin/perl
#
# CHESS.PL beta 1
# Copyright (C) 1997 John Watson
# email: john@watson-net.com
# Deutsche bersetzung Gerold Bigorajski
# email: Bigorajski@T-Online.de
#
# April 9, 1997
#
# -----About-----
# Jan-97: Got the barebones program working.  Only understood coordinate-coordinate
#         move notation (e.g. d3-e4).  Did not check for legal moves.
# Feb-97: Added code to support multiple simultaneous games.  Changed loadgame(),
#         savegame(), newgameform().  Added listgames().  Added additional text
#         datafile of games.
# 1-Mar-97: Added code to understand algebraic notation.  Also checking for legal
#           moves.  Implemented en-passant captures.
# 15-Mar-97: Finished code for 'lost right to castle'.  Now moving a rook or king
#            loses right to castle.  Implemented pawn promotion (e.g. b8/Q, fxg1/R).
# Nov-01: Bugfix for not unique draw from different rows.
#
# To do: Check for check, checkmate, stalemate, draw (50 move, 3 position repetition),
#        temporarily can't castle due to threat.
#
# 50 move draw: count moves since last pawn move.  if 50 then draw.
# 3 position repetition: store each position in an array in FEN.  After each move,
#   check for 3 duplicates.  If 3 dups then draw.
#
# The latest copy of this script and documentation can be obtained from
# http://www.watson-net.com/
#
# -----Distribution-----
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

use CGI;
$q = new CGI;

init();

# Debug-Ausgabe (1 = Ja, 0 = Nein)
$DEBUG = 0;

# Meldung des Tages? (1 = Zeigen, 0 = Weglassen)
$showmotd = 0;

# Standardwerte
$gamedir ='/vserv/g/gb-dire1/htdocs/publik/spiele/schach2/';
$gameurl = 'http://www.gb-direkt.de/publik/spiele/schach2/';
$progurl = '../../../cgi-bin/schach/';
$motd = $gamedir . 'motd.txt';
$gamelist =  $gamedir . 'chess.txt';
$program = $progurl . 'schach2.cgi';
$helpfile = $gameurl . 'hilfe.htm';
$width = 32;
$height = 30;

$bodytag = "<body background=\"http://www.gb-direkt.de/_themes/sumipntg/sumtextb.jpg\" bgcolor=#FFFFFF text=#808080 link=#3333CC vlink=#666699 alink=#990099><font face=\"verdana, arial, helvetica\">";
$separator = "<img src=\"http://www.gb-direkt.de/_themes/sumipntg/sumhorsa.gif\" width=\"300\" height=\"10\"></p>";
$titlecolor = "#003366";

$mailprog = '/usr/sbin/sendmail -t';

# Globale Variable
@movelist = ();
@capturelist = ();

# Steine
%blank   = ('light',$gameurl . 'light.gif', 'dark',$gameurl . 'dark.gif');
%wking   = ('light',$gameurl . 'wkl.gif', 'dark',$gameurl . 'wkd.gif');
%wqueen  = ('light',$gameurl . 'wql.gif', 'dark',$gameurl . 'wqd.gif');
%wrook   = ('light',$gameurl . 'wrl.gif', 'dark',$gameurl . 'wrd.gif');
%wbishop = ('light',$gameurl . 'wbl.gif', 'dark',$gameurl . 'wbd.gif');
%wknight = ('light',$gameurl . 'wnl.gif', 'dark',$gameurl . 'wnd.gif');
%wpawn   = ('light',$gameurl . 'wpl.gif', 'dark',$gameurl . 'wpd.gif');
%bking   = ('light',$gameurl . 'bkl.gif', 'dark',$gameurl . 'bkd.gif');
%bqueen  = ('light',$gameurl . 'bql.gif', 'dark',$gameurl . 'bqd.gif');
%brook   = ('light',$gameurl . 'brl.gif', 'dark',$gameurl . 'brd.gif');
%bbishop = ('light',$gameurl . 'bbl.gif', 'dark',$gameurl . 'bbd.gif');
%bknight = ('light',$gameurl . 'bnl.gif', 'dark',$gameurl . 'bnd.gif');
%bpawn   = ('light',$gameurl . 'bpl.gif', 'dark',$gameurl . 'bpd.gif');

# Deutsche Bezeichnungen
%germ_name = ('S' , 'N' , 'L' , 'B' , 'T' , 'R' , 'D' , 'Q');


# Beginn des Hauptprogramms
# Parameter einlesen
$move = $q->param('move');
$newwhite = $q->param('newwhite');
$newblack = $q->param('newblack');
$whitemail = $q->param('whitemail');
$blackmail = $q->param('blackmail');
$gamefile = $q->param('gamefile');
$password = $q->param('password');
$comment = $q->param('comment');

# Variable fuers Zuruecknehmen setzen
$takeback = $q->param('takeback');
if ($takeback eq 'Zurcknehmen')
{
	$takeback = 1;
}
else
{
	$takeback = 0;
}

# set flip board variable
$flipboard = $q->param('flipboard');
$flip = $q->param('flip');

# set flip board variable
if ($flipboard eq 'Brett umdrehen')
{
	if ($flip)
	{
		$flip = 0;
	}
	else
	{
		$flip = 1;
	}
}

# HTML-Header
header();

# Falls $newwhite oder $newblack -Parameter vorhanden sind: Neues Spiel starten
if ($newwhite || $newblack)
{
	init();
	$whiteplayer = $newwhite;
	$blackplayer = $newblack;
	if ($blackplayer eq "")
	{
		print "<h3>Sie haben keinen Namen fr Schwarz eingegeben.</h3>";
		exit;
	}
	if ($whiteplayer eq "")
	{
		print "<h3>Sie haben keinen Namen fr Wei eingegeben.</h3>";
		exit;
	}
	if ($password eq "")
	{
		print "<h3>Sie mssen ein Kennwort eingeben.</h3>";
		exit;
	}
	$gamefile = newgamefile();
	$gamepass = $password;
	savegame();
}

# Falls gamefile nicht gesetzt ist, neues Spielformular senden
if ($gamefile eq "")
{
	newform();
	exit;
}

# Existierendes Spiel laden
loadgame();

# Wenn der Benutzer ein gueltiges Kennwort eigegeben hat, ist er ein 'Spieler'
if ($gamepass eq $password)
{
	$player = 1;
}
else
{
	$player = 0;
}

# Wenn der Spielestatus nicht 'Begonnen' ist, dann Beobachtungsmodus
if ($gamestatus ne 'Begonnen')
{
	$player = 0;
}

# Zug zuruecknehmen
if ($takeback && $player)
{
	pop @movelist;
	savegame();
	init();
	loadgame();
}

# Wenn es keinen Namen fuer den weissen oder schwarzen Spieler gibt, erfragen
if ($whiteplayer eq '' || $blackplayer eq '')
{
	newform();
	exit;
}

# Wenn $move ein gueltiger Zug ist, durchfuehren und das Spiel sichern
if ($move ne '' && $player)
{
	$movestatus = movepiece($move);
	if ($movestatus == 0)
	{
		print "<p align=center><b><font color=#FF0000>Ungltiger Zug\!</font></b></p>\n";
	}
	if ($movestatus == -1)
	{
		print "<p align=center><b><font color=#FF0000>Der Zug ist nicht eindeutig\!</font></b></p>\n";
	}
	if ($movestatus == 1)
	{
		savegame();
		if ((tomove() eq 'w' && $whitemail ne '') ||
		   (tomove() eq 'b' && $blackmail ne ''))
		{
			mailmessage();
		}
	}
}

# Brett zeigen
displayboard();

# HTML-Fuss
footer();

exit;


sub header
{
	print $q->header;

	my $hilfe = $gameurl . 'hilfe.htm';
	print <<EOT;
<html>
<head>
<title>Schach</title>
</head>
$bodytag
<table width="100%">
	<tr>
		<td width="47\%" valign="top"><h2><font color="$titlecolor"><b>Schachspiel</b></font color></h2></td>
		<td valign="middle"><a target="_blank" href="$helpfile">Hilfe</a></td>
		<td valign="middle" align="right"><a target="_self" href="javascript:history.back()">Zurck</a></td>
	</tr>
</table>
EOT
}


sub footer
{
	print <<EOT;
</body>
</html>
EOT
}


sub newform
{
	#print $q->header();
    
	# Meldung des Tages
	if (-e $motd  &&  $showmotd == 1)
	{
		print "<h3>Meldung des Tages</h3>";
		open (FILE,"<$motd");
		print <FILE>;
		close(FILE);
	}
    
	# Spieleformular beginnen
	print <<EOT;
<h3><font color="$titlecolor">Existierende Spiele</font></h3>
<form action="$program" method="POST">
Whlen Sie ein vorhandenes Spiel:
EOT

	listgames();

	print <<EOT;
<br>
Kennwort (erforderlich, um Zge durchfhren zu knnen): 
	<input type="password" name="password"><br>
	<input type="submit" value="Spielen/Beobachten">
</form>

<h3><font color="$titlecolor">Neues Spiel beginnen:</font></h3>
<form action="$program" method="POST">
<table>
	<tr>
		<th width="145" align="right">Wei: </th>
		<td><input type="text" name="newwhite"></td>
	</tr>
	<tr>
		<th align="right">email: </th>
		<td><input type="text" name="whitemail"><br></td>
	</tr>
	<tr>
		<th align="right">Schwarz: </th>
		<td><input type="text" name="newblack"></td>
	</tr>
	<tr>
		<th align="right">email: </th>
		<td><input type="text" name="blackmail"></td>
	</tr>
	<tr>
		<th align="right">Kennwort: </th>
		<td><input type="password" name="password"></td>
	</tr>
	<tr>
		<td align="right"><input type="submit" value="Neues Spiel"></td>
		<td><input type="reset" value="   Lschen   "></td>
	</tr>
</table>
</form>
</body>
</html>
EOT
}


sub moveform
{
	print <<EOT0;
<form action="$program" method="POST">
	<input type="hidden" name="gamefile" value="$gamefile">
	<input type="hidden" name="password" value="$password">
	<input type="hidden" name="flip" value="$flip">
<table>
	<tr>
EOT0

	if ($player)
	{    
		print <<EOT1;
		<th align="right">Ihr Zug:</th>
		<td><input type="text" size="10" name="move"></td>
		<td><input type="submit" value="Ziehen"></td>
	</tr>
	<tr>
		<th align="right">Kommentar: </th>
		<td colspan="2"><input size="32" type="text" name="comment"></td>
	</tr>
	<tr>
		<td colspan="3"><input type="reset" value="   Lschen   "> 
		<input type="submit" name="takeback" value="Zurcknehmen"> 
EOT1
	}
	else
	{
		print "		<td>";
	}

	print <<EOT2;
		<input type="submit" name="flipboard" value="Brett umdrehen"></td>
	</tr>
</table>
</form>
<form action="$program" method="POST">
<table width="100\%">
	<tr>
		<td valign="top" align="center"><input type="submit" value="Neu beginnen"></td>
	</tr>
</table>
</form>
EOT2
}


sub newgamefile
{
	my $gamename = ();
    
	srand();
    
	CREATEGAMEFILENAME:
	$gamefile = substr($whiteplayer, 0, 2);
	$gamefile .= substr($blackplayer, 0, 2);
	$gamefile =~ s/(\ \ |\.|;|:|'|"|<|>)//ig;
	$gamefile .= int(rand(1)*10000)+1;
	$gamefile .= ".txt";

	if (-e $gamefile)
	{
		goto CREATEGAMEFILENAME;
	}

	open(GAME,">>$gamelist");
	print GAME "$gamefile\n";
	close(GAME);
    
	return $gamefile;
}


sub listgames
{
	my ($file, $white, $black, $status, $i) = ();
	my @list = ();
    
	$i = 0;
	open(GAME,"<$gamelist");
	while (!eof(GAME))
	{
		chop($file = <GAME>);
		open(GAME2,"<$gamedir$file");
		chop($status = <GAME2>);
		chop($junk = <GAME2>);
		chop($junk = <GAME2>);
		chop($white = <GAME2>);
		chop($junk = <GAME2>);
		chop($black = <GAME2>);
		close(GAME2);
		$list[$i] = "$status\t$white\t$black\t$file";
		$i++;
	}
	close(GAME);
    
	# In absteigender Reihenfolge sortieren
	@list = sort {$b cmp $a} @list;
    
	print '<select name="gamefile">';
	foreach (@list)
	{
		($status, $white, $black, $file) = split(/\t/, $_);
		print "<option value=\"$file\">($status) $white - $black</option>";
	}
	print "</select>";
}


sub loadgame
{
	@movelist = ();
	@capturelist = ();
	my ($i, $line) = ();
    
	open(GAME,"<$gamedir$gamefile") || die "Konnte Spieledatei nicht ffnen. Bitte noch einmal versuchen";
	chop($gamestatus = <GAME>);
	chop($gamedate = <GAME>);
	chop($gamepass = <GAME>);
	chop($whiteplayer = <GAME>);
	chop($whitemail = <GAME>);
	chop($blackplayer = <GAME>);
	chop($blackmail = <GAME>);
	chop($lastmove = <GAME>);
    
	while (!eof(GAME))
	{
		chop($line = <GAME>);
		movepiece($line);
	}
    
	close(GAME);
}


sub savegame
{
	local ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst,$date);   
	local (@days, @months);

	@days = ('Sonntag','Montag','Dienstag','Mittwoch','Donnerstag','Freitag','Samstag');
	@months = ('Januar','Februar','Mrz','April','Mai','Juni','Juli','August','September','Oktober','November','Dezember');

	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
  
	if ($hour < 10)
	{
		$hour = "0$hour";
	}
	if ($min < 10) 
	{
		$min = "0$min";
	}
	if ($sec < 10)
	{
		$sec = "0$sec";
	}
	$mon++;

	$lastmove = "$mday.$mon.$year - $hour\:$min\:$sec";

	open(GAME, ">$gamedir$gamefile");
	print GAME "$gamestatus\n$gamedate\n$gamepass\n$whiteplayer\n$whitemail\n$blackplayer\n$blackmail\n$lastmove\n";
	if ($#movelist >= 0)
	{
		foreach (@movelist)
		{
			print GAME "$_\n";
		}
	}
	close(GAME);
}


sub mailmessage
{
	# Uebergabe der Parameter an das Sendmail-Programm:

	$MAIL_PROGRAM = $mailprog;

	open (MAIL, "|$MAIL_PROGRAM");

	# Ausgabe des emails in die Datei MAIL.

	if (tomove() eq 'b')
	{
		print MAIL "To: $whitemail\n";
		print MAIL "From: $blackmail\n";
	}
	else
	{
		print MAIL "To: $blackmail\n";
		print MAIL "From: $whitemail\n";
	}

	# Anmerkung: Die Endmarke __STOP_OF_MAIL__ muss am Zeilenanfang stehen.

	print MAIL <<__STOP_OF_MAIL__;
Subject: Schachpartie $whiteplayer - $blackplayer
X-Priority: 1 (Highest)

Zge:

__STOP_OF_MAIL__

	$num = 1;
	for($i = 0; $i <= $#movelist; $i += 2)
	{
		print MAIL "$num. $movelist[$i] $movelist[$i+1]\n ";
		$num++;
	}
	print MAIL <<__STOP_OF_MAIL2__;

Letzter Zug: $move

Kommentar: $comment
__STOP_OF_MAIL2__

	if ($#movelist < 2)
	{
		print MAIL "Ihr Kennwort lautet: $password.\nBitte gut merken - bei zuknftigen Mails wird es nicht mitgeschickt!";
	}

	print MAIL <<__STOP_OF_MAIL3__;

Sie sind am Zug!

Mit freundlichen Gren
__STOP_OF_MAIL3__

	if (tomove() eq 'b')
	{
		print MAIL "$blackplayer\n";
	}
	else
	{
		print MAIL "$whiteplayer\n";
	}

	print MAIL "Das Internet-Schachbrett -\n$program";
	print MAIL "Ein Service von GB-Direkt!";

	close(MAIL);
}


sub tomove
{
	# Wer ist am Zug?
	if (($#movelist + 1)/2 == int(($#movelist + 1)/2))
	{
		return 'w';
	}
	else
	{
		return 'b';
	}
}


sub movepiece
{
	my $move = $_[0];
	my ($p, $s, $t, $f, $r, $promote, $iscapture, $annotation, $turn, $dir) = ();
	my @startsquares = ();
	my @legalmoves = ();
	my ($i, $j, $ix, $jx, $sr, $sf, $tr, $tf, $temp) = ();

	# Erklaerungen
	# $p = Name des Steins in englischer Notation (p, n, b, r, q, k)
	# $s = Startfeld
	# $t = Zielfeld
	# $f = Startlinie bei nicht eindeutigem Zug
	# $r = Startreihe bei nicht eindeutigem Zug
	# $promote = Der Stein, in den ein Bauer sich verwandeln moechte
	# $iscapture = Mit diesem Zug wurde ein Stein geschlagen
	# $annotation = jegliche Anmerkungen
	# $turn = Wer ist am Zug - 'w'hite oder 'b'lack
	# $dir = Bewegungs-Himmelsrichtung
	# @startsquares = enthaelt die Ergebnisse von piecepos()
	# @legalmoves = enthaelt alle als gueltig erkannten Zuege
	# $i, $j, $ix, $jx, $temp = Zaehlvariablen
	# $sr, $sf, $tr, $tf = Startreihe, Startlinie, Zielreihe, Ziellinie (temporaere Variable)
    
	# Wer ist dran?
	$turn = tomove();
    
	# Wenn im Zug ein x vorhanden ist, wurde ein Stein geschlagen
	if ($move =~ /x/i)
	{
		$iscapture = 1;
	}
	print "SCHLAGEN x\n" if $iscapture && $DEBUG == 1;

	# ZUG: AUFGEBEN
	if ($move =~ /Aufgeben/i)
	{
		if ($turn eq 'w')
		{
			$gamestatus = '0-1';
		}
		else
		{
			$gamestatus = '1-0';
		}
		push @movelist, $move;
		return 1;
	}
        
	# ZUG: KURZE ROCHADE (O-O)
	if (lc($move) eq "o-o"  ||  lc($move) eq "0-0")
	{
		print "Kurze Rochade\n" if $DEBUG == 1;
		if ($turn eq 'w' && $whitecastleshort)
		{
			return 0;
		}
		if ($turn eq 'b' && $blackcastleshort)
		{
			return 0;
		}
		# Weiss
		if ($turn eq 'w' && !$whitecastleshort)
		{
			if ($board{61} eq 'K' && $board{62} eq ' ' &&
				$board{63} eq ' ' && $board{64} eq 'R')
			{
				$board{61} = ' ';
				$board{62} = 'R';
				$board{63} = 'K';
				$board{64} = ' ';
				$whitecastleshort = 1;
			}
			else
			{
				return 0;
			}
		}
		# Schwarz
		if ($turn eq 'b' && !$blackcastleshort)
		{
			if ($board{5} eq 'k' && $board{6} eq ' ' &&
				$board{7} eq ' ' && $board{8} eq 'r')
			{
				$board{5} = ' ';
				$board{6} = 'r';
				$board{7} = 'k';
				$board{8} = ' ';
				$blackcastleshort = 1;
			}
			else
			{
				return 0;
			}
		}
	}
    
	# ZUG: LANGE ROCHADE (O-O-O)
	if (lc($move) eq "o-o-o"  ||  lc($move) eq "0-0-0")
	{
		print "Lange Rochade\n" if $DEBUG == 1;
		if ($turn eq 'w' && $whitecastlelong)
		{
			return 0;
		}
		if ($turn eq 'b' && $blackcastlelong)
		{
			return 0;
		}
		# Weiss
		if ($turn eq 'w' && !$whitecastlelong)
		{
			if ($board{57} = 'R' && $board{58} eq ' ' &&
				$board{59} eq ' ' && $board{60} eq ' ' &&
				$board{61} eq 'K')
			{
				$board{57} = ' ';
				$board{58} = ' ';
				$board{59} = 'K';
				$board{60} = 'R';
				$board{61} = ' ';
				$whitecastlelong = 1;
			}
			else
			{
				return 0;
			}
		}
		# Schwarz
		if ($turn eq 'b' && !$blackcastlelong)
		{
			if ($board{1} = 'r' && $board{2} eq ' ' &&
				$board{3} eq ' ' && $board{4} eq ' ' &&
				$board{5} eq 'k')
			{
				$board{1} = ' ';
				$board{2} = ' ';
				$board{3} = 'k';
				$board{4} = 'r';
				$board{5} = ' ';
				$blackcastlelong = 1;
			}
			else
			{
				return 0;
			}
		}
	}
    
	# ZUG: ROCHADE, DAHER DEN REST UEBERSPRINGEN
	# Wir waeren nicht hier, wenn die Rochade nicht erfolgreich gewesen waere, daher ...
	if (lc($move) eq "o-o" || lc($move) eq "o-o-o"  ||  lc($move) eq "0-0"  ||  lc($move) eq "0-0-0")
	{
		push @movelist, uc($move);
		return 1;
	}
    
	# BAUERNZUG (beginnt mit a|b|c|d|e|f|g|h)
	if ($move =~ /^[a-h]/)
	{
		print "Bauernzug\n" if $DEBUG == 1;
		$p = 'P';
		$f = substr($move,0,1);
		if ($iscapture)
		{
			# Schlagen dxe4
			$t = substr($move,2,2);
			# Umwandlung mit Schlagen dxe8/D
			$promote = $germ_name{substr($move,5,1)}  if ($move =~ m@(/S|/L|/T|/D)@);
		}
		else
		{
			# einfacher Zug e4
			$t = substr($move,0,2);
			# Umwandlung e8/D
			$promote = $germ_name{substr($move,3,1)}  if ($move =~ m@(/S|/L|/T|/D)@);
		}
	}
	# SPRINGER, LAEUFER, TURMZUG (beginnt mit S|L|T und length > 2)
	if ($move =~ /^S/ && length($move) > 2)
	{
		$p = 'N';
	}
	if ($move =~ /^L/ && length($move) > 2)
	{
		$p = 'B';
	}
	if ($move =~ /^T/ && length($move) > 2)
	{
		$p = 'R';
	}
	if ($move =~ /^[SLT]/ && length($move) > 2)
	{
		print "Springer, Lufer, Turmzug\n" if $DEBUG == 1;
		if (!$iscapture && length($move) == 3)
		{
			print "Kurznotation\n" if $DEBUG == 1;
			$f = "";
			$t = substr($move,1,2);
		}
		if (length($move) == 4)
		{
			print "Lange Notation oder Schlagen\n" if $DEBUG == 1;
			$f = substr($move,1,1);
			if ($f eq "x") { $f = ""; }
			if ($f =~ /[1-8]/)
			{
				# Es wurde die Startreihe, nicht die Linie angegeben
				$r = $f;
				$f = "";
			}
			$t = substr($move,2,2);
		}
		if (length($move) == 5)
		{
			print "Lange Notation plus Schlagen\n" if $DEBUG == 1;
			$f = substr($move,1,1);
			if ($f =~ /[1-8]/)
			{
				# Es wurde die Startreihe, nicht die Linie angegeben
				$r = $f;
				$f = "";
			}
			$t = substr($move,3,2);
		}
	}

	# DAME, KOENIG (beginnt mit d|k)
	if ($move =~ /^D/)
	{
		$p = 'Q';
	}
	if ($move =~ /^K/)
	{
		$p = 'K';
	}
	if ($move =~ /^[DK]/)
	{
		print "Dame, Knigszug\n" if $DEBUG == 1;
		$f = "";
		if ($iscapture)
		{
			$t = substr($move,2,2);
		}
		else
		{
			$t = substr($move,1,2);
		}
	}
    
	# Sicherstellen, dass das Zielfeld auf dem Brett ist
	print "Startreihe: $f\n" if $DEBUG == 1;
	if ($t =~ /[a-h][1-8]/)
	{
		print "Ziel ist auf dem Brett: $t\n" if $DEBUG == 1;
	}
	else
	{
		print "Ziel auerhalb des Bretts: $t\n" if $DEBUG == 1;
		return 0;
	}
    
	# Zielfeld in Feldnummer umwandeln
	$t = square(substr($t,0,1), substr($t,1,1));
	print "Zielfeld ist: $t\n" if $DEBUG == 1;
    
	# ALLE ERLAUBTEN ZGE FINDEN
	if ($turn eq 'w')
	{
		@startsquares = piecepos(uc($p));
	}
	else
	{
		@startsquares = piecepos(lc($p));
	}
    
	if ($DEBUG == 1)
	{
		print "Startfelder: ";
		foreach(@startsquares)
		{
			print "$_ ";
		}
		print "\n";
	}
    
	# Schleife durch alle Startfelder, um Uebereinstimmung zu finden
	foreach (@startsquares)
	{
		($sf, $sr) = rankfile($_); # Startreihe und -linie
		($tf, $tr) = rankfile($t); # Zielreihe und -linie
        
		# Wenn diese Startreihe nicht die vom Spieler spezifizierte ist, naechster Schleifendurchgang
		if (($f && ($sf ne $f)) || ($r && ($sr ne $r)))
		{
			next;
		}
        
		# Wenn auf $targetsquare ein Stein steht und es sich nicht um ein Schlagen handelt, 
		# Ende der Schleife
		if ($board{$t} =~ /[pnbrqk]/i && !$iscapture)
		{
			last;
		}
        
		# Wenn auf $targetsquare kein Stein steht und es sich um ein Schlagen handelt, Ende der Schleife
		if ($board{$t} !~ /[pnbrqk]/i && $iscapture && $enpassant != $t)
		{
			last;
		}
        
		# Wenn dies ein Bauernzug ist ...
		if ($p eq 'P')
		{
			# WENN NICHT (Bauer bewegt sich nur um ein Feld vorwaerts ODER 
			# um zwei Felder von der Grundreihe ODER er schlaegt einen Stein)
			# DANN naechster Schleifendurchgang.
			if (!(abs($file{$sf}-$file{$tf}) == 0 && abs($sr-$tr) == 1 ||
				abs($file{$sf}-$file{$tf}) == 0 && abs($sr-$tr) == 2 &&
				($sr==2 && $turn eq 'w' || $sr==7 && $turn eq 'b') ||
				abs($file{$sf}-$file{$tf}) == 1 && abs($sr-$tr) == 1))
			{
				next;
			}
			# WENN (Versuch der Umwandlung und kein Zug auf die letzte Reihe) DANN naechster.
			if ($promote ne '' && ($turn eq 'w' && $tr != 8 || $turn eq 'b' && $tr != 1))
			{
				next;
			}
			# WENN (Zug auf die letzte Reihe und keine Umwandlung) DANN naechster.
			if ($promote eq '' && ($turn eq 'w' && $tr == 8 || $turn eq 'b' && $tr == 1))
			{
				next;
			}
		}
        
		# Wenn es ein Koenigszug ist ...
		if ($p eq 'K')
		{
			# WENN NICHT (Bewegung um ein Feld) DANN naechster
 			if (!(abs($file{$sf}-$file{$tf}) == 0 && abs($sr-$tr) == 1 ||
				abs($file{$sf}-$file{$tf}) == 1 && abs($sr-$tr) == 0 ||
				abs($file{$sf}-$file{$tf}) == 1 && abs($sr-$tr) == 1))
			{
				next;
			}
			# Rochaderecht geht verloren
			if ($turn eq 'w')
			{
				$whitecastleshort = 1;
				$whitecastlelong = 1;
			}
			else
			{
				$blackcastleshort = 1;
				$blackcastlelong = 1;
			}               
		}
        
		# Wenn dies ein Turmzug ist ...
		if ($p eq 'R')
		{
			# Bei erster Bewegung vom Grundfeld geht das Rochaderecht auf dieser Seite verloren
			if ($_ == 1)
			{
				$blackcastlelong = 1;
			}
			if ($_ == 8)
			{
				$blackcastleshort = 1;
			}
			if ($_ == 57)
			{
				$whitecastlelong = 1;
			}
			if ($_ == 64)
			{
				$whitecastleshort = 1;
			}
		}
        
		# Wenn dies ein Springerzug ist ...
		if ($p eq 'N')
		{
			# ... und der Zug wirklich gekruemmt ist ...
			if (abs($file{$sf}-$file{$tf}) == 1 && abs($sr-$tr) == 2 ||
				abs($file{$sf}-$file{$tf}) == 2 && abs($sr-$tr) == 1)
			{
				# ... und der Spieler nicht versucht, seine eigenen Steine zu schlagen ...
				if ($turn eq 'w' && $board{$t} =~ /[pnbrqk\ ]/)
				{
					push(@legalmoves, "$_,$t");
				}
				if ($turn eq 'b' && $board{$t} =~ /[PNBRQK\ ]/)
				{
					push(@legalmoves, "$_,$t");
				}
			}
		}
		else
		{
			# Dies ist kein Springerzug, daher die Zugrichtung ermitteln ...
			$dir = movedir($_, $t);
			print "Zugrichtung: $dir\n" if $DEBUG == 1;
			# ... und die Zugrichtung in Inkremente umwandeln ...
			if ($dir eq 'n')
			{
				$ix =  0;
				$jx =  1;
			}
			if ($dir eq 's')
			{
				$ix =  0;
				$jx = -1;
			}
			if ($dir eq 'e')
			{
				$ix =  1;
				$jx =  0;
			}
			if ($dir eq 'w')
			{
				$ix = -1;
				$jx =  0;
			}
			if ($dir eq 'ne')
			{
				$ix =  1;
				$jx =  1;
			}
			if ($dir eq 'nw')
			{
				$ix = -1;
				$jx =  1;
			}
			if ($dir eq 'se')
			{
				$ix =  1;
				$jx = -1;
			}
			if ($dir eq 'sw')
			{
				$ix = -1;
				$jx = -1;
			}
            
			# Gueltige Richtungen fuer Tuerme/Laeufer/Bauern ueberpruefen
			if ($p eq 'R' && ($dir eq 'ne' || $dir eq 'nw' || $dir eq 'se' || $dir eq 'sw'))
			{
				next;
			}
			if ($p eq 'B' && ($dir eq 'n' || $dir eq 's' || $dir eq 'e' || $dir eq 'w'))
			{
				next;
			}
			if ($p eq 'P' && ($turn eq 'w' && ($dir eq 'e' || $dir eq 'w' || $dir eq 'se' || $dir eq 'sw' || $dir eq 's') ||
				$turn eq 'b' && ($dir eq 'e' || $dir eq 'w' || $dir eq 'ne' || $dir eq 'nw' || $dir eq 'n')))
			{
				next;
			}

			print "  ix: $ix  jx: $jx\n" if $DEBUG == 1;
			# ... initialisieren der Startlinie und -reihe ...
			$i = $file{$sf};
			$j = $sr;
			$temp = 0;
			# ... Schleife durch die Felder, bis das Zielfelderreicht ist oder das Brett verlassen wird oder
			# ein anderer Stein angetroffen wird.
			while (($i != $file{$tf} || $j != $tr) && $i >= 1 && $i <= 8 && $j >= 1 && $j <= 8)
			{
				$i = $i + $ix;
				$j = $j + $jx;
				$temp = square($filen{$i},$j);
				print "Testfeld: $temp\n" if $DEBUG == 1;
				if ($board{square($filen{$i},$j)} =~ /[pnbrqk]/i)
				{
					last;
				}
			}
			# Wenn das erreichte Feld unser Zielfeld ist ...
			print "Feld erreicht: $temp\n" if $DEBUG == 1;
			if ($temp eq $t)
			{
				# ... und der Spieler nicht versucht, seine eigenen Steine zu schlagen.
				if ($turn eq 'w' && $board{$t} =~ /[pnbrqk\ ]/)
				{
					push(@legalmoves, "$_,$t");
				}
				if ($turn eq 'b' && $board{$t} =~ /[PNBRQK\ ]/)
				{
					push(@legalmoves, "$_,$t");
				}
			}
		}
	}
    
	if ($DEBUG == 1)
	{
		print "Gltige Zge gefunden:\n";
		foreach(@legalmoves)
		{
			print "  $_\n";
		}
	}
    
	# Mehrdeutiger Zug (mehr als ein gueltiger Zug)
	if ($#legalmoves > 0)
	{
		return -1;
	}
    
	# ZUG ZUR ZUGLISTE HINZUFUEGEN UND ERFOLG ZURUECKGEBEN
	if ($#legalmoves == 0)
	{
		($s, $t) = split(/,/, $legalmoves[0]);
		($sf, $sr) = rankfile($s);
		($tf, $tr) = rankfile($t);
		$p = $board{$s};
        
		# Normales Schlagen
		if ($board{$t} ne ' ')
		{
			push @capturelist, $board{$t};
		}
        
		# Schlagen en passant
		if ($enpassant == $t)
		{
			if ($turn eq 'w')
			{
				push @capturelist, 'p'; $board{$t + 8} = ' ';
			}
			if ($turn eq 'b')
			{
				push @capturelist, 'P'; $board{$t - 8} = ' ';
			}
		}
        
		if ($turn eq 'w')
		{
			$p = uc($p);
		}
		else
		{
			$p = lc($p);
		}

		if ($promote ne '')
		{
			if ($turn eq 'w')
			{
				$p = uc($promote);
			}
			else
			{
				$p = lc($promote);
			}
		}
        
		$board{$t} = $p;
		$board{$s} = " ";
		push @movelist, $move;

		# Wenn der Bauer zwei Felder zog, wird das en passant-Flag gesetzt, sonst
		# wird es abgestellt
		if (lc($p) eq 'p' && abs($sr-$tr) == 2)
		{
			if ($turn eq 'w')
			{
				$enpassant = $s - 8;
			}
			if ($turn eq 'b')
			{
				$enpassant = $s + 8;
			}
		}
		else
		{
			$enpassant = 0;
		}

		return 1;
	}
}


sub movedir
{
	# Rueckgabe der Richtung bei gegebenem Start- und Endfeld:
	# (Angabe als Himmelsrichtung)
	#
	# nw n ne
	#   \|/
	#  w-+-e
	#   /|\
	# sw s se
	#

	my ($s, $t) = @_;
	my ($sf, $sr, $tf, $tr) = ();
    
	($sf, $sr) = rankfile($s);
	($tf, $tr) = rankfile($t);

	# Norden
	if ($tr > $sr && $file{$sf} == $file{$tf})
	{
		return 'n';
	}

	#Sueden
	if ($tr < $sr && $file{$sf} == $file{$tf})
	{
		return 's';
	}

	# Osten
	if ($sr == $tr && $file{$tf} > $file{$sf})
	{
		return 'e';
	}

	# Westen
	if ($sr == $tr && $file{$tf} < $file{$sf})
	{
		return 'w';
	}

	# Nordosten
	if ($tr > $sr && $file{$tf} > $file{$sf})
	{
		return 'ne';
	}

	# Suedosten
	if ($tr < $sr && $file{$tf} > $file{$sf})
	{
		return 'se';
	}

	# Suedwesten
	if ($tr < $sr && $file{$tf} < $file{$sf})
	{
		return 'sw';
	}

	# Nordwesten
	if ($tr > $sr && $file{$tf} < $file{$sf})
	{
		return 'nw';
	}
    
	# Start- und Endpunkt sind identisch?
	return 0;
}


sub piecepos
{
	# Rueckgabe des Feldes, auf dem ein bestimmter Stein sich befindet
	my $piece = $_[0];
	my @squares = ();

	for($i=1;$i<=64;$i++)
	{
		if ($board{$i} eq $piece)
		{
			push(@squares, $i);
		}
	}
    
	return(@squares);
}


sub displayboard
{
	my ($i, $j, $k, $c, $p, $file, $rank, $num);
    
	print "<table><tr>\n";
	# Zeigen des Brettes
	print "<td valign=top>";
	print "<center><font color=#CC3300>";

	# Name der Spieler oben auf dem Brett
	if ($flip)
	{
		print "<b>$whiteplayer</b>";
	}
	else
	{
		print "<b>$blackplayer</b>";
	}

	print "</font></center>";
	print "<table border=1><tr>\n";

	$i = 1;
	$k = 0;
	while ($i<=64)
	{
		if ($flip)
		{
			$j = 64 - $i + 1;
		}
		else
		{
			$j = $i;
		}
        
		$c = color($j);
		$p = $board{$j};
		($file, $rank) = rankfile($j);
        
		print "<td width=$width><img width=$width height=$height alt=\"$p\" src=\"";
		print "$blank{$c}" if ($p eq ' ');
		print "$bking{$c}" if ($p eq 'k');
		print "$bqueen{$c}" if ($p eq 'q');
		print "$brook{$c}" if ($p eq 'r');
		print "$bbishop{$c}" if ($p eq 'b');
		print "$bknight{$c}" if ($p eq 'n');
		print "$bpawn{$c}" if ($p eq 'p');
		print "$wking{$c}" if ($p eq 'K');
		print "$wqueen{$c}" if ($p eq 'Q');
		print "$wrook{$c}" if ($p eq 'R');
		print "$wbishop{$c}" if ($p eq 'B');
		print "$wknight{$c}" if ($p eq 'N');
		print "$wpawn{$c}" if ($p eq 'P');
		print "\"></td>\n";

		$k++;

		if ($k/8 == int($k/8))
		{
			print "<td width=$width>&nbsp;&nbsp;$rank&nbsp;&nbsp;</td></tr>\n<tr>";
			$k = 0;
		}
		$i++;
	}
	print "</tr>\n<tr>";

	if ($flip)
	{
		print qq~
	<td height=30 align=center width=$width>h</td>
	<td align=center width=$width>g</td>
	<td align=center width=$width>f</td>
	<td align=center width=$width>e</td>
	<td align=center width=$width>d</td>
	<td align=center width=$width>c</td>
	<td align=center width=$width>b</td>
	<td align=center width=$width>a</td>~;
	}
	else
	{
		print qq~
	<td height=30 align=center width=$width>a</td>
	<td align=center width=$width>b</td>
	<td align=center width=$width>c</td>
	<td align=center width=$width>d</td>
	<td align=center width=$width>e</td>
	<td align=center width=$width>f</td>
	<td align=center width=$width>g</td>
	<td align=center width=$width>h</td>~;
	}
	print "\n</tr></table>\n";
	print "<center><font color=#CC3300>";

	# Spielername unten am Brett
	if ($flip)
	{
		print "<b>$blackplayer</b>";
	}
	else
	{
		print "<b>$whiteplayer</b>";
	}

	print "</font></center>";

	# Zugformular
	moveform();

	print "</td>\n";

	# Kleiner Zwischenraum, dann Zug- & Schlagliste
	print qq~
		<td width=10 valign=top>\&nbsp\;<td>
		<td valign=top>
		<b><i><u>Partie</u>:</i></b>  <font size=1>[$gamefile]</font size><br>
		<b>$whiteplayer - $blackplayer</b><br>
	~;

	if ($gamestatus eq 'Begonnen')
	{
		if (tomove() eq 'w')
		{
			print "<b>Wei am Zug.</b>\n";
		}
		else
		{
			print "<b>Schwarz am Zug.</b>\n";
		}
	}

	print "[Beobachtungsmodus]\n" if !$player;

	print "<p><b>Geschlagene Steine</b><br>\n";
	@capturelist = sort @capturelist;
	foreach (@capturelist)
	{
		print "<img width=",$width/1.5," height=",$height/1.5," src=\"";
		print "$bking{'light'}" if ($_ eq 'k');
		print "$bqueen{'light'}" if ($_ eq 'q');
		print "$brook{'light'}" if ($_ eq 'r');
		print "$bbishop{'light'}" if ($_ eq 'b');
		print "$bknight{'light'}" if ($_ eq 'n');
		print "$bpawn{'light'}" if ($_ eq 'p');
		print "$wking{'light'}" if ($_ eq 'K');
		print "$wqueen{'light'}" if ($_ eq 'Q');
		print "$wrook{'light'}" if ($_ eq 'R');
		print "$wbishop{'light'}" if ($_ eq 'B');
		print "$wknight{'light'}" if ($_ eq 'N');
		print "$wpawn{'light'}" if ($_ eq 'P');
		print "\">\n";
	}
	print "</p><p>\n";

	if ($comment ne '')
	{
		print qq~
<b><i>Kommentar:</i></b> <font color="#008844" size="1">$comment</font><br>~;
	}

	print qq~
<b><i>Zge</i> <font size=1>(Letzter Zug: $lastmove - <font color=#FF0000>$movelist[$#movelist]</font>)</font></b>
<pre><table><tr>~;

	$num = 1;
	for($i = 0; $i <= $#movelist; $i += 2)
	{
		$num = ' ' . $num if (length($num) == 1);
		$wzug = substr($movelist[$i] . '      ' , 0, 6);
		$szug = substr($movelist[$i+1] . '      ' , 0, 6);
		print "<td><font color=#000000 size=2><b>$num.</b> $wzug $szug </font></td>\n";
		print "</tr><tr>"  if (($i+2)/6 == int(($i+2)/6));
		$num++;
	}
	print "</tr></table></pre>";

	print "</td>\n";
	print "</tr></table>\n";
}


sub color
{
	# Rueckgabe der Farbe, wenn die Feldnummer gegeben ist.
	# Gerade beginnt mit hellen Feldern.
	# Ungerade beginnt mit dunklen Feldern.
	my $square = $_[0];
	my ($f, $r) = rankfile($square);
	my $diff = $square - (64 - $r*8 + 1);
	my $color;
    
	if ($r/2 == int($r/2))
	{
		$color = 2 + $diff;
	}
	else
	{
		$color = 1 + $diff;
	}
    
	if ($color/2 == int($color/2))
	{
		return 'light';
	}
	else
	{
		return 'dark';
	}
}


sub square
{
	# Rueckgabe der Feldnummer bei gegebener Linie und Reihe
	my ($f, $r) = @_;
	return($file{$f} + (8-$r)*8);
}


sub rankfile
{
	# Rueckgabe der Reihe und Linie einer gegebenen Feldnummer
	my $square = $_[0];
	my ($r, $f, $diff) = ();
    
	$r = 1  if ($square <=64);
	$r = 2  if ($square <=56);
	$r = 3  if ($square <=48);
	$r = 4  if ($square <=40);
	$r = 5  if ($square <=32);
	$r = 6  if ($square <=24);
	$r = 7  if ($square <=16);
	$r = 8  if ($square <= 8);
    
	$diff = 72 - $r * 8 - $square + 1;
	$f = 'a'  if ($diff == 8);
	$f = 'b'  if ($diff == 7);
	$f = 'c'  if ($diff == 6);
	$f = 'd' if ($diff == 5);
	$f = 'e'  if ($diff == 4);
	$f = 'f'  if ($diff == 3);
	$f = 'g'  if ($diff == 2);
	$f = 'h'  if ($diff == 1);
    
	return($f, $r);
}


sub init
{
	%file = ('a', '1', 'b', '2', 'c', '3', 'd', '4', 'e', '5', 'f', '6', 'g', '7', 'h', '8');

	%filen = ('1', 'a', '2', 'b', '3', 'c', '4', 'd', '5', 'e', '6', 'f', '7', 'g', '8', 'h');
    
  %board = ( '1', 'r',  '2', 'n',  '3', 'b',  '4', 'q', '5', 'k',  '6', 'b',  '7', 'n',  '8', 'r',
		 '9', 'p', '10', 'p', '11', 'p', '12', 'p', '13', 'p', '14', 'p', '15', 'p', '16', 'p',
		'17', ' ', '18', ' ', '19', ' ', '20', ' ', '21', ' ', '22', ' ', '23', ' ', '24', ' ',
		'25', ' ', '26', ' ', '27', ' ', '28', ' ', '29', ' ', '30', ' ', '31', ' ', '32', ' ',
		'33', ' ', '34', ' ', '35', ' ', '36', ' ', '37', ' ', '38', ' ', '39', ' ', '40', ' ',
		'41', ' ', '42', ' ', '43', ' ', '44', ' ', '45', ' ', '46', ' ', '47', ' ', '48', ' ',
		'49', 'P', '50', 'P', '51', 'P', '52', 'P', '53', 'P', '54', 'P', '55', 'P', '56', 'P',
		'57', 'R', '58', 'N', '59', 'B', '60', 'Q', '61', 'K', '62', 'B', '63', 'N', '64', 'R');

	@movelist = ();
	@capturelist = ();

	$whitecastleshort = 0;
	$whitecastlelong = 0;
	$blackcastleshort = 0;
	$blackcastlelong = 0;

	# Setzen, ob Schlagen en passant moegich ist
	# 0 = Schlagen en passant ist nicht moeglich
	$enpassant = 1;
    
	# Status des Spiels und Startzeit
	$gamestatus = 'Begonnen';
	$gamedate = localtime();
}

